using System;
using System.Threading;
using InfiniTec.WebDAV;
using Monitor = InfiniTec.WebStorageSystem.Monitor;

namespace SubsriptionDemo
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Class1
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			Monitor monitor;
			IWebDAVFolder root;
			Connection conn;


			monitor = null;
			try 
			{
				if (args.Length < 3) throw new ArgumentException("Not enough parameters");
				conn = new Connection(new Credential());
				conn.Server = args[0];
				conn.Protocol = ConnectionProtocol.Http;
				root = conn.AddRoot(args[1]);

				monitor = new Monitor(root);

				monitor.ItemChanged += new EventHandler(monitor_ItemChanged);

				monitor.Subscribe(args[2]);

				Console.Out.WriteLine("Monitoring " + root.Uri);
				while (true)
				{
					Thread.Sleep(1000);
					Console.Out.Write(".");
				}

			}
			catch (ArgumentException ex)
			{
				ShowUsage();
				Console.Out.WriteLine(ex.Message);
			}
			catch (Exception ex)
			{
				Console.Out.WriteLine("The following error occured: " + ex.Message);
			}
			finally
			{
				if (monitor != null) monitor.RemoveSubscription();
			}

		}

		private static void ShowUsage()
		{
			Console.Out.WriteLine("Subscription example.");
			Console.Out.WriteLine("Usage: subscriptionexample <server> <path> <type>");
			Console.Out.WriteLine("Where: ");
			Console.Out.WriteLine("  <server>\tspecifies the server you want to monitor");
			Console.Out.WriteLine("  <path>\tspecifies the path to be monitored (ex. public/myfolder)");
			Console.Out.WriteLine("  <type>\tsThe notification type (update, update/newmember, delete,\r\n\t\tmove, or pragma/<http://schemas.microsoft.com/exchange/newmail>)");
			Console.Out.WriteLine();
		}

		private static void monitor_ItemChanged(object sender, EventArgs e)
		{
			Console.Out.WriteLine("\r\nThe item has changed!");
		}
	}
}
