using System;
using System.Timers;

using InfiniTec.WebDAV;

namespace InfiniTec.WebStorageSystem
{
	/// <summary>
	/// Summary description for Monitor.
	/// </summary>
	public class Monitor
	{
		private const int SubscriptionDuration = 3600; // One hour

		public event EventHandler ItemChanged;

		private static UDPListener _Listener;

		private static int _Port;

		string _SubscriptionId;
		IWebDAVElement _Element;
		WebDAVLayer _WebDavLayer;
		Guid _Id;
		Timer _Timer;


		public static int Port 
		{
			get { return _Port; }
			set { _Port = value; }
		}

		protected virtual void OnItemChanged() 
		{
			if (ItemChanged != null) ItemChanged(this, new EventArgs());
		}

		public Guid Id 
		{
			get { return _Id;}
		}

		public IWebDAVElement Element 
		{
			get 
			{
				return _Element;
			}
		}

		public Monitor(IWebDAVElement element)
		{
			if (_Listener == null) 
			{
				_Listener = new UDPListener(_Port);
			}

			_Listener.UdpPacketReceived += new UdpPacketReceivedEventHandler(Listener_UdpPacketReceived);

			_Id = Guid.NewGuid();
			_Timer = new Timer(SubscriptionDuration / 2);
			_Element = element;
			
			if (element is WebDAVElement) 
			{
				_WebDavLayer = ((WebDAVElement) _Element).WebDAVLayer;
			}
			else
			{
				_WebDavLayer = ((WebDAVFolder) _Element).WebDAVLayer;
			}
		
			
			_Timer.Elapsed += new ElapsedEventHandler(_Timer_Elapsed);
		}

		public void Subscribe(string notificationType) 
		{
			lock (_WebDavLayer) 
			{
				_SubscriptionId =  _WebDavLayer.Subscribe(_Element.Uri, notificationType, (_Element is WebDAVFolder) ? 1 : 0, string.Format("httpu://{0}:{1}/{2}", System.Net.Dns.GetHostName(), _Listener.Port, _Id), SubscriptionDuration);
				_Timer.Start();
			}
		}

		public void RemoveSubscription() 
		{
			lock (_WebDavLayer) 
			{
				_WebDavLayer.UnSubscribe(_Element.Uri, _SubscriptionId);
				_Timer.Stop();
			}
		}

		private void _Timer_Elapsed(object sender, ElapsedEventArgs e)
		{
			int interval;

			lock (_WebDavLayer) 
			{
				try 
				{
					if (_WebDavLayer.Poll(_Element.Uri, _SubscriptionId).Length > 0) 
					{
						OnItemChanged();
						interval = SubscriptionDuration / 2;
					}
				}
				catch
				{
					interval = (int) _Timer.Interval / 2;
					if (interval < 1000) interval = 1000;
				}	
			}
		}

		public void Poll() 
		{
			if (_WebDavLayer.Poll(_Element.Uri, _SubscriptionId).Length > 0) 
			{
				OnItemChanged();
			}
		}
		private void Listener_UdpPacketReceived(object sender, UdpPacketReceivedEventArgs e)
		{
			if (e.Uri.EndsWith(_Id.ToString())) OnItemChanged();
		}
	}
}
