using System;
using System.Collections.Specialized;
using System.Net;
using System.Net.Sockets;

namespace InfiniTec.WebStorageSystem
{
	/// <summary>
	/// Summary description for UDPListener.
	/// </summary>
	public class UDPListener
	{
		Socket _Socket;

		public event UdpPacketReceivedEventHandler UdpPacketReceived;

		protected virtual void OnUdpPacketReceived(UdpPacketReceivedEventArgs e) 
		{
			if (UdpPacketReceived != null) UdpPacketReceived(this, e);
		}

		public int Port 
		{
			get { return ((IPEndPoint) _Socket.LocalEndPoint).Port; }
		}

		public UDPListener(int port)
		{
			IPHostEntry ipHostInfo;
			IPEndPoint localEP;

			_Socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);

			ipHostInfo = Dns.Resolve(Dns.GetHostName());
			localEP = new IPEndPoint(ipHostInfo.AddressList[0], port);
			
			_Socket.Bind(localEP);
		}

		public void Start() 
		{
			byte[] buffer;

			buffer = new byte[16384];

			_Socket.BeginReceive(buffer, 0, buffer.Length, SocketFlags.None, new AsyncCallback(Socket_EndReceive), buffer);
		}

		private void Socket_EndReceive(IAsyncResult ar) 
		{
			string message;
			string[] headerLines;
			string[] headerEntry;
			string path;
			string headerLine;
			byte[] buffer;
			int i;

			NameValueCollection headers;

			i = _Socket.EndReceive(ar);
			buffer = (byte[]) ar.AsyncState;

			if (i != 0) 
			{
				message = System.Text.UTF8Encoding.UTF8.GetString(buffer, 0, i);

				// Header Collection initialisieren
				headers = new NameValueCollection();

				// Empfangene Daten in Zeilen aufteilen
				headerLines = message.Split('\n');

				// Den bertragenen Pfad extrahieren.
				path = headerLines[0];
				path = path.Remove(path.LastIndexOf(" "), path.Length - path.LastIndexOf(" "));
				path = path.Remove(0, path.LastIndexOf("/")+1);

				// Die Header-Informationen extrahieren
				i=1;
				// Iterieren, bis entweder Ende der Daten oder eine leere Zeile entdeckt wurde.
				while ((i<headerLines.Length) && (headerLines[i].Trim() != "")) 
				{
					// Die aktuelle Zeile "subern"
					headerLine = headerLines[i].Trim(new char[] {'\r', ' '}); 
					// In Name und Wert Paar splitten.
					headerEntry = headerLine.Split(new char[] {':'});
					// In Header Kollektion einfgen
					headers.Add(headerEntry[0].Trim(), headerEntry[1].Trim());
					i++;
				}

				// Erneut auf Daten warten.
				_Socket.BeginReceive(buffer, 0, buffer.Length, SocketFlags.None, new AsyncCallback(Socket_EndReceive), buffer);
			
				// Event feuern.
				OnUdpPacketReceived(new UdpPacketReceivedEventArgs(path, headers));
			}
		}
	}
}
